# 2021-06-14 @ BLR
# Building the Shashlyk ... in the tetragonal unit cell - tetragonal space group : P 42/n [#56]
#

# Update LOG
#
# 2021-06-14 - Bug found by Dr. Alexander Filatov (U. Chicago) and corrected today.
#
# 2021-03-07 - spacer added in GSA output file
# 2021-03-06 - Clean up time
#
# 2021-02-17 - RES / SHELX / MERCURY format ( a4,a4,i1,4f10.6,f9.6 ) ... name , spacer4 , specie , x , y , z , osf , uiso
# 2021-02-17 - GSA / GSAS format ( a2,4f10.6,a6,f9.6 ) ... type , x ,y ,z , occ , name , uiso
#        

# 2021-02-04 - DMBDT WING 2
#
# 2021-02-03 - Addition of a .GSA / .ATO file for GSA
# 2021-02-03 - Addition of a .RES file for visualization with MERCURY / CSD / CCDC
#
# 2021-02-03 - DMBDT WING 1
# 2021-01-02 - FE4S4 CLUSTER

# Imported Libraries
import numpy as np
import matplotlib.pyplot as plt
import datetime

# Constants
spacer  = ' '
spacer4 = '    '
# SHLX constants
osf  = 1.00000
uiso = 0.00000
#

# S-S = f( Fe-Fe , Fe - s ) - My notebook 3 (2021) p. 173
#
# d1 = Fe-Fe ; d2 = Fe-S
#
def ss_distance(d1,d2):
  delta = 8.0 * ( 3.0 * d2**2 - d1**2 )
  x     = ( d1 + np.sqrt(delta) ) / 3.0
  return (x)

# Open .LOG file
#
filog = open('shashlyk_v63.log','w')

# Open .GSA file [ my former .ATO format ]
#
filout_gsa = open('shashlyk_v63.gsa','w')

# Open .RES Shelx file [ for MERCURY / CSD / CCDC visualization ]
#
filout_res = open('shashlyk_v63.res','w')

# RES file header
#
res_header_1  = 'TITL P42UN_18_phase_1'
res_header_2  = 'CELL 0.45793 23.8190 23.8190 20.7652 90 90 90'
res_header_3  = 'ZERR 8 0.0005 0.0000 0.0004 0.0 0.0 0.0'
res_header_4  = 'LATT 1'
res_header_5  = 'SYMM -y,1/2+x,1/2+z'
res_header_6  = 'SYMM 1/2-x,1/2-y,z'
res_header_7  = 'SYMM 1/2+y,-x,1/2+z'
res_header_8  = 'SFAC C H N S Fe'
res_header_9  = 'UNIT 384 704 8 64 32'
res_header_10 = 'FVAR 1.00'
#
filout_res.write(res_header_1 + '\n')
filout_res.write(res_header_2 + '\n')
filout_res.write(res_header_3 + '\n')
filout_res.write(res_header_4 + '\n')
filout_res.write(res_header_5 + '\n')
filout_res.write(res_header_6 + '\n')
filout_res.write(res_header_7 + '\n')
filout_res.write(res_header_8 + '\n')

# SET DATE AND TIME
starttime = str(datetime.datetime.now())[:19]  # truncated after 20 characters ...
print()
print(starttime)
print()
#
filog.write(            '\n')
filog.write(starttime + '\n')
filog.write(            '\n')
#
#
# FRONT-END HEADER
print()
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *   WELCOME ... to our software ... FE4S4DMBDT2_MOGUL ... [v 063 - 2021-06-14]   * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *    Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]    * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *                            Robert.Papoular@cea.fr                              * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print()
#
filog.write(                                                                                         '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *   WELCOME ... to our software ... FE4S4DMBDT2_MOGUL ... [v 063 - 2021-06-14]   * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *    Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]    * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *                            Robert.Papoular@cea.fr                              * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(                                                                                         '\n')
#

#
# Lattice parameters a , b , c ... as found using a LeBail fit
# Tetragonal cell - tetragonal space group P 42/n [#56]
#
# All distances in Angstroms
#
a = 23.8190   # tetragonal unit cell
b = 23.8190   # tetragonal unit cell
c = 20.7652   # tetragonal unit cell
#
filog.write('\n')
filog.write('UNIT CELL LATTICE PARAMETERS: \n')
filog.write('\n')
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] a = % 10.5f A \n' %(a)) 
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] b = % 10.5f A \n' %(b))
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] c = % 10.5f A \n' %(c))
filog.write('\n')
filog.write('\n')

def dot_product(X1,Y1,Z1,X2,Y2,Z2):
  dot = X1*X2*a*a + Y1*Y2*b*b + Z1*Z2*c*c
  return dot

def cross_product(X1,Y1,Z1,X2,Y2,Z2):
  cross_x = ( Y1*Z2 - Z1*Y2 )*(b*c/a)
  cross_y = ( Z1*X2 - X1*Z2 )*(c*a/b)
  # cross_z = ( X1*Y2 - Y2*X1 )*(a*b/c)          # ERROR FOUND by Dr. A. Filatov (U. Chicago) 2021-06-14
  cross_z = ( X1*Y2 - Y1*X2 )*(a*b/c)            # Corrected   by R.J.P.                      2021-06-14
  return cross_x , cross_y , cross_z

def rotation(x1,y1,z1,x2,y2,z2,x3,y3,z3,angle):  # rotation axis : M1M2 , angle : theta [degrees ], M3 ===> M4
                                                 # M2M3 ===> M2M4
  angler = angle * np.pi / 180.0
  # AB
  u1 = x2 - x1
  v1 = y2 - y1
  w1 = z2 - z1
  # BC
  u2 = x3 - x2
  v2 = y3 - y2
  w2 = z3 - z2
  #
  ABdotBC = dot_product(u1,v1,w1,u2,v2,w2)
  #                      
  ABcrossBC_x , ABcrossBC_y , ABcrossBC_z = cross_product(u1,v1,w1,u2,v2,w2)
  #
  u1 = x2 - x1
  v1 = y2 - y1
  w1 = z2 - z1
  #
  u2 = x2 - x1
  v2 = y2 - y1
  w2 = z2 - z1
  #
  ABdotAB = dot_product(u1,v1,w1,u2,v2,w2)
  #
  x4 = x2 + (x3 - x2)   *       np.cos(angler)
  x4 = x4 + (x2 - x1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  x4 = x4 + ABcrossBC_x *       np.sin(angler)           / np.sqrt(ABdotAB)
  #
  y4 = y2 + (y3 - y2)   *       np.cos(angler)
  y4 = y4 + (y2 - y1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  y4 = y4 + ABcrossBC_y *       np.sin(angler)           / np.sqrt(ABdotAB) 
  #
  z4 = z2 + (z3 - z2)   *       np.cos(angler)
  z4 = z4 + (z2 - z1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  z4 = z4 + ABcrossBC_z *       np.sin(angler)           / np.sqrt(ABdotAB)
  return x4 , y4 , z4

# PART I: BUILD ... FE4S4 ... CLUSTER
#
print()
print('PART I: ... FE4S4 CLUSTER')
print()
#
filog.write('PART I: ... FE4S4 CLUSTER \n')

#
# Center of Fe-S cluster
# ... in crystallographic coordinates
#
x_center = 0.00   # tetragonal unit cell - tetragonal space group  
y_center = 0.00   # tetragonal unit cell - tetragonal space group
z_center = 0.25   # tetragonal unit cell - tetragonal space group
#
filog.write('\n')
filog.write('FE4S4 CLUSTER CENTER :')
filog.write('\n')
filog.write('[ FE4S4 center ] x_center = % 10.5f \n' %(x_center)) 
filog.write('[ FE4S4 center ] y_center = % 10.5f \n' %(y_center))
filog.write('[ FE4S4 center ] z_center = % 10.5f \n' %(z_center))
filog.write('\n')

# Fe-Fe, distance .... from MOGUL / CSD 2020 / CCDC 
#
fefe = 2.768
#
x_fe = fefe/np.sqrt(8.0)
y_fe = x_fe
z_fe = x_fe
#
x_fe = x_fe/a
y_fe = y_fe/b
z_fe = z_fe/c
#
x_fe_1 = x_center + x_fe
y_fe_1 = y_center + y_fe
z_fe_1 = z_center + z_fe
#
x_fe_2 = x_center - x_fe
y_fe_2 = y_center - y_fe
z_fe_2 = z_center + z_fe
#
#
x_fe_3 = x_center + x_fe
y_fe_3 = y_center - y_fe
z_fe_3 = z_center - z_fe
#
x_fe_4 = x_center - x_fe
y_fe_4 = y_center + y_fe
z_fe_4 = z_center - z_fe
#
x_fe_check = (x_fe_1 + x_fe_2 + x_fe_3 + x_fe_4)/4.0
y_fe_check = (y_fe_1 + y_fe_2 + y_fe_3 + y_fe_4)/4.0
z_fe_check = (z_fe_1 + z_fe_2 + z_fe_3 + z_fe_4)/4.0
#
#
print()
print('FE atoms crystallographics coordinates ... :')
print('  Fe1 ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f' %(x_fe_1,y_fe_1,z_fe_1))
print('  Fe2 ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f' %(x_fe_2,y_fe_2,z_fe_2))
print('  Fe3 ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f' %(x_fe_3,y_fe_3,z_fe_3))
print('  Fe4 ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f' %(x_fe_4,y_fe_4,z_fe_4))
print()
print('Check : crystallographic coordinates of Fe-Tetrahedron center ... : % 10.5f % 10.5f % 10.5f' %(x_fe_check,y_fe_check,z_fe_check))
print()
#
#

# 2021-02-17 - GSA / GSAS format ( a2,4f10.6,a6,f9.6 ) ... type , x ,y ,z , occ , name , uiso
# 2021-02-17 - RES / SHELX / MERCURY format ( a4,a4,i1,4f10.6,f9.6 ) ... name , spacer4 , specie , x , y , z , osf , uiso

# Save ... FE ... result in GSA & XYZ ouput files
#     filog.write('[FE4S4 CLUSTER: FE1 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_1,y_fe_1,z_fe_1))
name1 = 'FE1'
type1 = 'FE'
sfac1 = 5
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type1.ljust(2),x_fe_1,y_fe_1,z_fe_1,osf,spacer,name1.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name1.ljust(4),spacer4,sfac1,x_fe_1,y_fe_1,z_fe_1,osf,uiso))
#
name2 = 'FE2'
type2 = 'FE'
sfac2 = 5
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type2.ljust(2),x_fe_2,y_fe_2,z_fe_2,osf,spacer,name2.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name2.ljust(4),spacer4,sfac2,x_fe_2,y_fe_2,z_fe_2,osf,uiso))
#
name3 = 'FE3'
type3 = 'FE'
sfac3 = 5
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type3.ljust(2),x_fe_3,y_fe_3,z_fe_3,osf,spacer,name3.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name3.ljust(4),spacer4,sfac3,x_fe_3,y_fe_3,z_fe_3,osf,uiso))
#
name4 = 'FE4'
type4 = 'FE'
sfac4 = 5
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type4.ljust(2),x_fe_4,y_fe_4,z_fe_4,osf,spacer,name4.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name4.ljust(4),spacer4,sfac4,x_fe_4,y_fe_4,z_fe_4,osf,uiso))

#
#
#
# S-S, distance .... from MOGUL / CSD 2020 / CCDC ... via Fe-S distance 
#
fes = 2.272  # from MOGUL 
ss = ss_distance(fefe,fes)

print(                                  )
print('ss ... [ in A ] ... %10.5f' %(ss))
print(                                  )

#
x_s = ss/np.sqrt(8.0)
y_s = x_s
z_s = x_s
#
x_s = x_s/a
y_s = y_s/b
z_s = z_s/c
#
x_s_5 = x_center + x_s
y_s_5 = y_center - y_s
z_s_5 = z_center + z_s
#
x_s_6 = x_center - x_s
y_s_6 = y_center + y_s
z_s_6 = z_center + z_s
#
#
x_s_7 = x_center + x_s
y_s_7 = y_center + y_s
z_s_7 = z_center - z_s
#
x_s_8 = x_center - x_s
y_s_8 = y_center - y_s
z_s_8 = z_center - z_s
#
x_s_check = (x_s_5 + x_s_6 + x_s_7 + x_s_8)/4.0
y_s_check = (y_s_5 + y_s_6 + y_s_7 + y_s_8)/4.0
z_s_check = (z_s_5 + z_s_6 + z_s_7 + z_s_8)/4.0
#
#
print()
print('S  atoms crystallographics coordinates ... :')
print('  S5  ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f ' %(x_s_5,y_s_5,z_s_5))
print('  S6  ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f ' %(x_s_6,y_s_6,z_s_6))
print('  S7  ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f ' %(x_s_7,y_s_7,z_s_7))
print('  S8  ... from Fe4S4 cluster : % 10.5f % 10.5f % 10.5f ' %(x_s_8,y_s_8,z_s_8))
print()
print('Check : crystallographic coordinates of S-Tetrahedron center  ... : % 10.5f % 10.5f % 10.5f \n' %(x_s_check,y_s_check,z_s_check))
print()
#
#
# name4 = 'FE4'
# type4 = 'FE'
# sfac4 = 5
# filout_gsa.write('%2s %10.6f %10.6f %10.6f %10.6f %6s %9.6f\n' %(type4.ljust(2),x_fe_4,y_fe_4,z_fe_4,osf,name4.ljust(6),uiso))
# filout_res.write('%4s %4s %1d %10.6f %10.6f %10.6f %10.6f %9.6f\n' %(name4.ljust(4),spacer4,sfac4,x_fe_4,y_fe_4,z_fe_4,osf,uiso))
#
# Save ... S ... Fe4S4 ... cluster ... result in GSA & XYZ ouput files
#     filog.write('[FE4S4 CLUSTER: FE1 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_1,y_fe_1,z_fe_1))
name5 = 'S5'
type5 = 'S'
sfac5 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type5.ljust(2),x_s_5,y_s_5,z_s_5,osf,spacer,name5.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name5.ljust(4),spacer4,sfac5,x_s_5,y_s_5,z_s_5,osf,uiso))
#
name6 = 'S6'
type6 = 'S'
sfac6 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type6.ljust(2),x_s_6,y_s_6,z_s_6,osf,spacer,name6.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name6.ljust(4),spacer4,sfac6,x_s_6,y_s_6,z_s_6,osf,uiso))
#
name7 = 'S7'
type7 = 'S'
sfac7 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type7.ljust(2),x_s_7,y_s_7,z_s_7,osf,spacer,name7.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name7.ljust(4),spacer4,sfac7,x_s_7,y_s_7,z_s_7,osf,uiso))
#
name8 = 'S8'
type8 = 'S'
sfac8 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type8.ljust(2),x_s_8,y_s_8,z_s_8,osf,spacer,name8.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name8.ljust(4),spacer4,sfac8,x_s_8,y_s_8,z_s_8,osf,uiso))

# PART II: BUILD ... DMBDT ... WING 1
#
print()
print('PART II: ... DMBDT ... WING 1')
print()


# More distances ...
#
r         = 1.400 # Benzene ring - perfect hexagon
sc        = 1.761 # S - C covalent bond [MERCURY/CSD/CCDC]
cc_single = 1.506 # C - C covalent single bond
ch_sp3    = 1.000 # C - H sp3
hh_sp3    = 1.633 # H - H sp3
#
d1 = c * ( 1.0 - 2.0 * z_fe_1 ) # FE1-FE2star distance
d2 = 2.0 * ( r + sc )           # S9-S10 distance
d3 = fes                        # FE - S covalent bond distance ( from above )
#
p = 0.5 * ( d1 - d2 )
q = np.sqrt( d3*d3 - p*p )
#
x_s_9  = x_fe_1 + ( q / (a*np.sqrt(2)) )
y_s_9  = y_fe_1 + ( q / (a*np.sqrt(2)) )
z_s_9  = z_fe_1 + ( p / c )
#
x_s_10 = x_s_9
y_s_10 = y_s_9
z_s_10 = z_s_9  + ( d2 / c )
#
x_c_11 = x_s_9
y_c_11 = y_s_9
z_c_11 = z_s_9  + ( sc / c )
#
x_c_12 = x_s_10
y_c_12 = y_s_10
z_c_12 = z_s_10 - ( sc / c )
#
x_c_13 = x_c_11 +       ( r / a )*np.sqrt(3/8)
y_c_13 = y_c_11 +       ( r / a )*np.sqrt(3/8)
z_c_13 = z_c_11 + 0.5 * ( r / c )
#
x_c_14 = x_c_11 -       ( r / a )*np.sqrt(3/8)
y_c_14 = y_c_11 -       ( r / a )*np.sqrt(3/8)
z_c_14 = z_c_11 + 0.5 * ( r / c )
#
x_c_15 = x_c_13
y_c_15 = y_c_13
z_c_15 = z_c_13 +       ( r / c )
#
x_c_16 = x_c_14
y_c_16 = y_c_14 
z_c_16 = z_c_14 +       ( r / c )
#
x_c_17 = x_c_15 + np.sqrt(3.0/8.0) * ( cc_single / a )
y_c_17 = y_c_15 + np.sqrt(3.0/8.0) * ( cc_single / a )
z_c_17 = z_c_15 +         0.5      * ( cc_single / c )
#
x_c_18 = x_c_14 - np.sqrt(3.0/8.0) * ( cc_single / a )
y_c_18 = y_c_14 - np.sqrt(3.0/8.0) * ( cc_single / a )
z_c_18 = z_c_14 -         0.5      * ( cc_single / c )
#
x_h_19 = x_c_13 + np.sqrt(3.0/8.0) * ( ch_sp3 / a )
y_h_19 = y_c_13 + np.sqrt(3.0/8.0) * ( ch_sp3 / a )
z_h_19 = z_c_13 -         0.5      * ( ch_sp3 / c )
#
x_h_20 = x_c_16 - np.sqrt(3.0/8.0) * ( ch_sp3 / a )
y_h_20 = y_c_16 - np.sqrt(3.0/8.0) * ( ch_sp3 / a )
z_h_20 = z_c_16 +         0.5      * ( ch_sp3 / c )
# 
x_h_21 = x_c_17 + ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
y_h_21 = y_c_17 + ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
z_h_21 = z_c_17 - ( np.sqrt(2.0/3.0) -         1.0/ 6.0 ) * ( ch_sp3 / c )
# 
x_h_22 = x_c_18 - ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
y_h_22 = y_c_18 - ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
z_h_22 = z_c_18 + ( np.sqrt(2.0/3.0) -         1.0/ 6.0 ) * ( ch_sp3 / c )
#
x_h_23 , y_h_23 , z_h_23 = rotation(x_c_15,y_c_15,z_c_15,x_c_17,y_c_17,z_c_17,x_h_21,y_h_21,z_h_21, 120)
x_h_24 , y_h_24 , z_h_24 = rotation(x_c_15,y_c_15,z_c_15,x_c_17,y_c_17,z_c_17,x_h_21,y_h_21,z_h_21,-120)
x_h_25 , y_h_25 , z_h_25 = rotation(x_c_14,y_c_14,z_c_14,x_c_18,y_c_18,z_c_18,x_h_22,y_h_22,z_h_22, 120)
x_h_26 , y_h_26 , z_h_26 = rotation(x_c_14,y_c_14,z_c_14,x_c_18,y_c_18,z_c_18,x_h_22,y_h_22,z_h_22,-120)

#
# name4 = 'FE4'
# type4 = 'FE'
# sfac4 = 5
# filout_gsa.write('%2s %10.6f %10.6f %10.6f %10.6f %6s %9.6f\n' %(type4.ljust(2),x_fe_4,y_fe_4,z_fe_4,osf,name4.ljust(6),uiso))
# filout_res.write('%4s %4s %1d %10.6f %10.6f %10.6f %10.6f %9.6f\n' %(name4.ljust(4),spacer4,sfac4,x_fe_4,y_fe_4,z_fe_4,osf,uiso))
#
name9  = 'S9'
type9  = 'S'
sfac9  = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type9.ljust(2),x_s_9,y_s_9,z_s_9,osf,spacer,name9.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name9.ljust(4),spacer4,sfac9,x_s_9,y_s_9,z_s_9,osf,uiso))
#
name10 = 'S10'
type10 = 'S'
sfac10 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type10.ljust(2),x_s_10,y_s_10,z_s_10,osf,spacer,name10.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name10.ljust(4),spacer4,sfac10,x_s_10,y_s_10,z_s_10,osf,uiso))
#
name11 = 'C11'
type11 = 'C'
sfac11 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type11.ljust(2),x_c_11,y_c_11,z_c_11,osf,spacer,name11.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name11.ljust(4),spacer4,sfac11,x_c_11,y_c_11,z_c_11,osf,uiso))
#
name12 = 'C12'
type12 = 'C'
sfac12 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type12.ljust(2),x_c_12,y_c_12,z_c_12,osf,spacer,name12.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name12.ljust(4),spacer4,sfac12,x_c_12,y_c_12,z_c_12,osf,uiso))
#
name13 = 'C13'
type13 = 'C'
sfac13 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type13.ljust(2),x_c_13,y_c_13,z_c_13,osf,spacer,name13.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name13.ljust(4),spacer4,sfac13,x_c_13,y_c_13,z_c_13,osf,uiso))
#
name14 = 'C14'
type14 = 'C'
sfac14 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type14.ljust(2),x_c_14,y_c_14,z_c_14,osf,spacer,name14.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name14.ljust(4),spacer4,sfac14,x_c_14,y_c_14,z_c_14,osf,uiso))
#
name15 = 'C15 '
type15 = 'C'
sfac15 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type15.ljust(2),x_c_15,y_c_15,z_c_15,osf,spacer,name15.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name15.ljust(4),spacer4,sfac15,x_c_15,y_c_15,z_c_15,osf,uiso))
#
name16 = 'C16'
type16 = 'C'
sfac16 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type16.ljust(2),x_c_16,y_c_16,z_c_16,osf,spacer,name16.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name16.ljust(4),spacer4,sfac16,x_c_16,y_c_16,z_c_16,osf,uiso))
#
name17 = 'C17'
type17 = 'C'
sfac17 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type17.ljust(2),x_c_17,y_c_17,z_c_17,osf,spacer,name17.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name17.ljust(4),spacer4,sfac17,x_c_17,y_c_17,z_c_17,osf,uiso))
#
name18 = 'C18'
type18 = 'C'
sfac18 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type18.ljust(2),x_c_18,y_c_18,z_c_18,osf,spacer,name18.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name18.ljust(4),spacer4,sfac18,x_c_18,y_c_18,z_c_18,osf,uiso))
#
name19 = 'H19'
type19 = 'H'
sfac19 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type19.ljust(2),x_h_19,y_h_19,z_h_19,osf,spacer,name19.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name19.ljust(4),spacer4,sfac19,x_h_19,y_h_19,z_h_19,osf,uiso))
#
name20 = 'H20'
type20 = 'H'
sfac20 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type20.ljust(2),x_h_20,y_h_20,z_h_20,osf,spacer,name20.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name20.ljust(4),spacer4,sfac20,x_h_20,y_h_20,z_h_20,osf,uiso))
#
name21 = 'H21'
type21 = 'H'
sfac21 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type21.ljust(2),x_h_21,y_h_21,z_h_21,osf,spacer,name21.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name21.ljust(4),spacer4,sfac21,x_h_21,y_h_21,z_h_21,osf,uiso))
#
name22 = 'H22'
type22 = 'H'
sfac22 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type22.ljust(2),x_h_22,y_h_22,z_h_22,osf,spacer,name22.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name22.ljust(4),spacer4,sfac22,x_h_22,y_h_22,z_h_22,osf,uiso))
#
name23 = 'H23'
type23 = 'H'
sfac23 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type23.ljust(2),x_h_23,y_h_23,z_h_23,osf,spacer,name23.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name23.ljust(4),spacer4,sfac23,x_h_23,y_h_23,z_h_23,osf,uiso))
#
name24 = 'H24'
type24 = 'H'
sfac24 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type24.ljust(2),x_h_24,y_h_24,z_h_24,osf,spacer,name24.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name24.ljust(4),spacer4,sfac24,x_h_24,y_h_24,z_h_24,osf,uiso))
#
name25 = 'H25 '
type25 = 'H'
sfac25 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type25.ljust(2),x_h_25,y_h_25,z_h_25,osf,spacer,name25.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name25.ljust(4),spacer4,sfac25,x_h_25,y_h_25,z_h_25,osf,uiso))
#
name26 = 'H26 '
type26 = 'H'
sfac26 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type26.ljust(2),x_h_26,y_h_26,z_h_26,osf,spacer,name26.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name26.ljust(4),spacer4,sfac26,x_h_26,y_h_26,z_h_26,osf,uiso))


print()
print('DMBDT WING 1 ...  atomic crystallographics coordinates ... :')
print('  S9   ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_s_9 ,y_s_9 ,z_s_9 ))
print('  S10  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_s_10,y_s_10,z_s_10))
print('  C11  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_11,y_c_11,z_c_11))
print('  C12  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_12,y_c_12,z_c_12))
print('  C13  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_13,y_c_13,z_c_13))
print('  C14  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_14,y_c_14,z_c_14))
print('  C15  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_15,y_c_15,z_c_15))
print('  C16  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_16,y_c_16,z_c_16))
print('  C17  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_17,y_c_17,z_c_17))
print('  C18  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_c_18,y_c_18,z_c_18))
print('  H19  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_19,y_h_19,z_h_19))
print('  H20  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_20,y_h_20,z_h_20))
print('  H21  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_21,y_h_21,z_h_21))
print('  H22  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_22,y_h_22,z_h_22))
print('  H23  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_23,y_h_23,z_h_23))
print('  H24  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_24,y_h_24,z_h_24))
print('  H25  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_25,y_h_25,z_h_25))
print('  H26  ... from DMBDT 1 : % 10.5f % 10.5f % 10.5f ' %(x_h_26,y_h_26,z_h_26))
print()

# RESULTS :
#
filog.write('KEY DISTANCES from MOGUL / CSD 2020 / CCDC ... :')
filog.write('\n')
filog.write(' FE - FE distance : % 10.5f A \n' %(fefe))
filog.write(' FE - S  distance : % 10.5f A \n' %(fes))
filog.write(' S - S   distance : % 10.5f A \n' %(ss))
filog.write('\n')
#
filog.write('[FE4S4 CLUSTER] CRYSTALLOGRAPHIC COORDINATES OF ALL 4 ... FE ... ATOMS: ')
filog.write('\n')
filog.write('[FE4S4 CLUSTER: FE1 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_1,y_fe_1,z_fe_1))
filog.write('[FE4S4 CLUSTER: FE2 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_2,y_fe_2,z_fe_2))
filog.write('[FE4S4 CLUSTER: FE3 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_3,y_fe_3,z_fe_3))
filog.write('[FE4S4 CLUSTER: FE4 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_fe_4,y_fe_4,z_fe_4))
filog.write('\n')
filog.write('[FE4S4 CLUSTER] CRYSTALLOGRAPHIC COORDINATES OF ALL 4 ... S ... ATOMS: ')
filog.write('\n')
filog.write('[FE4S4 CLUSTER: S5  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_5,y_s_5,z_s_5))
filog.write('[FE4S4 CLUSTER: S6  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_6,y_s_6,z_s_6))
filog.write('[FE4S4 CLUSTER: S7  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_7,y_s_7,z_s_7))
filog.write('[FE4S4 CLUSTER: S8  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_8,y_s_8,z_s_8))
filog.write('\n')

#
filog.write('                              \n')
filog.write('PART II: ... DMBDT ... WING 1 \n')
filog.write('\n')
filog.write('[DMBDT WING 1] CRYSTALLOGRAPHIC COORDINATES OF ALL 2 ... S ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 1: S9   - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_9,y_s_9,z_s_9))
filog.write('[DMBDT WING 1: S10  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_10,y_s_10,z_s_10))
filog.write('\n')
filog.write('[DMBDT WING 1] CRYSTALLOGRAPHIC COORDINATES OF ALL 8 ... C ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 1: C11  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_11,y_c_11,z_c_11))
filog.write('[DMBDT WING 1: C12  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_12,y_c_12,z_c_12))
filog.write('[DMBDT WING 1: C13  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_13,y_c_13,z_c_13))
filog.write('[DMBDT WING 1: C14  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_14,y_c_14,z_c_14))
filog.write('[DMBDT WING 1: C15  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_15,y_c_15,z_c_15))
filog.write('[DMBDT WING 1: C16  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_16,y_c_16,z_c_16))
filog.write('[DMBDT WING 1: C17  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_17,y_c_17,z_c_17))
filog.write('[DMBDT WING 1: C18  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_18,y_c_18,z_c_18))
filog.write('\n')
filog.write('[DMBDT WING 1] CRYSTALLOGRAPHIC COORDINATES OF ALL 8 ... H ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 1: H19  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_19,y_h_19,z_h_19))
filog.write('[DMBDT WING 1: H20  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_20,y_h_20,z_h_20))
filog.write('[DMBDT WING 1: H21  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_21,y_h_21,z_h_21))
filog.write('[DMBDT WING 1: H22  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_22,y_h_22,z_h_22))
filog.write('[DMBDT WING 1: H19  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_23,y_h_23,z_h_23))
filog.write('[DMBDT WING 1: H24  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_24,y_h_24,z_h_24))
filog.write('[DMBDT WING 1: H25  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_25,y_h_25,z_h_25))
filog.write('[DMBDT WING 1: H26  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_26,y_h_26,z_h_26))
filog.write('\n')

# PART III: BUILD ... DMBDT ... WING 2
#
print()
print('PART III: ... DMBDT ... WING 2')
print()
#
filog.write('                               \n')
filog.write('PART III: ... DMBDT ... WING 2 \n')

#
x_s_27  = x_fe_3 + ( q / (a*np.sqrt(2)) )
y_s_27  = y_fe_3 - ( q / (a*np.sqrt(2)) )
z_s_27  = z_fe_3 - ( p / c )
#
x_s_28 = x_s_27
y_s_28 = y_s_27
z_s_28 = z_s_27 - ( d2 / c )
#
x_c_29 = x_s_27
y_c_29 = y_s_27
z_c_29 = z_s_27 - ( sc / c )
#
x_c_30 = x_s_28
y_c_30 = y_s_28
z_c_30 = z_s_28 + ( sc / c )
#
x_c_31 = x_c_29 +       ( r / a )*np.sqrt(3/8)
y_c_31 = y_c_29 -       ( r / a )*np.sqrt(3/8)
z_c_31 = z_c_29 - 0.5 * ( r / c )
#
x_c_32 = x_c_29 -       ( r / a )*np.sqrt(3/8)
y_c_32 = y_c_29 +       ( r / a )*np.sqrt(3/8)
z_c_32 = z_c_29 - 0.5 * ( r / c )
#
x_c_33 = x_c_31
y_c_33 = y_c_31
z_c_33 = z_c_31 -       ( r / c )
#
x_c_34 = x_c_32
y_c_34 = y_c_32 
z_c_34 = z_c_32 -       ( r / c )
#
#
x_c_35 = x_c_31 + np.sqrt(3.0/8.0) * ( cc_single / a )
y_c_35 = y_c_31 - np.sqrt(3.0/8.0) * ( cc_single / a )
z_c_35 = z_c_31 +         0.5      * ( cc_single / c )
#
x_c_36 = x_c_34 - np.sqrt(3.0/8.0) * ( cc_single / a )
y_c_36 = y_c_34 + np.sqrt(3.0/8.0) * ( cc_single / a )
z_c_36 = z_c_34 -         0.5      * ( cc_single / c )
#
#
x_h_37 = x_c_33 + np.sqrt(3.0/8.0) * ( ch_sp3 / a )
y_h_37 = y_c_33 - np.sqrt(3.0/8.0) * ( ch_sp3 / a )
z_h_37 = z_c_33 -         0.5      * ( ch_sp3 / c )
#
x_h_38 = x_c_32 - np.sqrt(3.0/8.0) * ( ch_sp3 / a )
y_h_38 = y_c_32 + np.sqrt(3.0/8.0) * ( ch_sp3 / a )
z_h_38 = z_c_32 +         0.5      * ( ch_sp3 / c )
# 
x_h_39 = x_c_35 + ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
y_h_39 = y_c_35 - ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
z_h_39 = z_c_35 - ( np.sqrt(2.0/3.0) -         1.0/ 6.0 ) * ( ch_sp3 / c )
# 
x_h_40 = x_c_36 - ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
y_h_40 = y_c_36 + ( np.sqrt(1.0/9.0) + np.sqrt(1.0/24.0)) * ( ch_sp3 / a )
z_h_40 = z_c_36 + ( np.sqrt(2.0/3.0) -         1.0/ 6.0 ) * ( ch_sp3 / c )
#
#
x_h_41 , y_h_41 , z_h_41 = rotation(x_c_31,y_c_31,z_c_31,x_c_35,y_c_35,z_c_35,x_h_39,y_h_39,z_h_39, 120)
x_h_42 , y_h_42 , z_h_42 = rotation(x_c_31,y_c_31,z_c_31,x_c_35,y_c_35,z_c_35,x_h_39,y_h_39,z_h_39,-120)
x_h_43 , y_h_43 , z_h_43 = rotation(x_c_34,y_c_34,z_c_34,x_c_36,y_c_36,z_c_36,x_h_40,y_h_40,z_h_40, 120)
x_h_44 , y_h_44 , z_h_44 = rotation(x_c_34,y_c_34,z_c_34,x_c_36,y_c_36,z_c_36,x_h_40,y_h_40,z_h_40,-120)


#
# name4 = 'FE4'
# type4 = 'FE'
# sfac4 = 5
# filout_gsa.write('%2s %10.6f %10.6f %10.6f %10.6f %6s %9.6f\n' %(type4.ljust(2),x_fe_4,y_fe_4,z_fe_4,osf,name4.ljust(6),uiso))
# filout_res.write('%4s %4s %1d %10.6f %10.6f %10.6f %10.6f %9.6f\n' %(name4.ljust(4),spacer4,sfac4,x_fe_4,y_fe_4,z_fe_4,osf,uiso))
#
name27 = 'S27'
type27 = 'S'
sfac27 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type27.ljust(2),x_s_27,y_s_27,z_s_27,osf,spacer,name27.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name27.ljust(4),spacer4,sfac27,x_s_27,y_s_27,z_s_27,osf,uiso))
#
name28 = 'S28'
type28 = 'S'
sfac28 = 4
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type28.ljust(2),x_s_28,y_s_28,z_s_28,osf,spacer,name28.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name28.ljust(4),spacer4,sfac28,x_s_28,y_s_28,z_s_28,osf,uiso))
#
name29 = 'C29'
type29 = 'C'
sfac29 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type29.ljust(2),x_c_29,y_c_29,z_c_29,osf,spacer,name29.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name29.ljust(4),spacer4,sfac29,x_c_29,y_c_29,z_c_29,osf,uiso))
#
name30 = 'C30'
type30 = 'C'
sfac30 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type30.ljust(2),x_c_30,y_c_30,z_c_30,osf,spacer,name30.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name30.ljust(4),spacer4,sfac30,x_c_30,y_c_30,z_c_30,osf,uiso))
#
name31 = 'C31'
type31 = 'C'
sfac31 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type31.ljust(2),x_c_31,y_c_31,z_c_31,osf,spacer,name31.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name31.ljust(4),spacer4,sfac31,x_c_31,y_c_31,z_c_31,osf,uiso))
#
name32 = 'C32'
type32 = 'C'
sfac32 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type32.ljust(2),x_c_32,y_c_32,z_c_32,osf,spacer,name32.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name32.ljust(4),spacer4,sfac32,x_c_32,y_c_32,z_c_32,osf,uiso))
#
name33 = 'C33'
type33 = 'C'
sfac33 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type33.ljust(2),x_c_33,y_c_33,z_c_33,osf,spacer,name33.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name33.ljust(4),spacer4,sfac33,x_c_33,y_c_33,z_c_33,osf,uiso))
#
name34 = 'C34'
type34 = 'C'
sfac34 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type34.ljust(2),x_c_34,y_c_34,z_c_34,osf,spacer,name34.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name34.ljust(4),spacer4,sfac34,x_c_34,y_c_34,z_c_34,osf,uiso))
#
name35 = 'C35'
type35 = 'C'
sfac35 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type35.ljust(2),x_c_35,y_c_35,z_c_35,osf,spacer,name35.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name35.ljust(4),spacer4,sfac35,x_c_35,y_c_35,z_c_35,osf,uiso))
#
name36 = 'C36'
type36 = 'C'
sfac36 = 1
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type36.ljust(2),x_c_36,y_c_36,z_c_36,osf,spacer,name36.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name36.ljust(4),spacer4,sfac36,x_c_36,y_c_36,z_c_36,osf,uiso))
#
name37 = 'H37 '
type37 = 'H'
sfac37 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type37.ljust(2),x_h_37,y_h_37,z_h_37,osf,spacer,name37.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name37.ljust(4),spacer4,sfac37,x_h_37,y_h_37,z_h_37,osf,uiso))
#
name38 = 'H38'
type38 = 'H'
sfac38 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type38.ljust(2),x_h_38,y_h_38,z_h_38,osf,spacer,name38.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name38.ljust(4),spacer4,sfac38,x_h_38,y_h_38,z_h_38,osf,uiso))
#
name39 = 'H39'
type39 = 'H'
sfac39 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type39.ljust(2),x_h_39,y_h_39,z_h_39,osf,spacer,name39.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name39.ljust(4),spacer4,sfac39,x_h_39,y_h_39,z_h_39,osf,uiso))
#
name40 = 'H40 '
type40 = 'H'
sfac40 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type40.ljust(2),x_h_40,y_h_40,z_h_40,osf,spacer,name40.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name40.ljust(4),spacer4,sfac40,x_h_40,y_h_40,z_h_40,osf,uiso))
#
name41 = 'H41'
type41 = 'H'
sfac41 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type41.ljust(2),x_h_41,y_h_41,z_h_41,osf,spacer,name41.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name41.ljust(4),spacer4,sfac41,x_h_41,y_h_41,z_h_41,osf,uiso))
#
name42 = 'H42'
type42 = 'H'
sfac42 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type42.ljust(2),x_h_42,y_h_42,z_h_42,osf,spacer,name42.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name42.ljust(4),spacer4,sfac42,x_h_42,y_h_42,z_h_42,osf,uiso))
#
name43 = 'H43'
type43 = 'H'
sfac43 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type43.ljust(2),x_h_43,y_h_43,z_h_43,osf,spacer,name43.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name43.ljust(4),spacer4,sfac43,x_h_43,y_h_43,z_h_43,osf,uiso))
#
name44 = 'H44'
type44 = 'H'
sfac44 = 2
filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(type44.ljust(2),x_h_44,y_h_44,z_h_44,osf,spacer,name44.ljust(6),uiso))
filout_res.write('%4s%4s%1d%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(name44.ljust(4),spacer4,sfac44,x_h_44,y_h_44,z_h_44,osf,uiso))


print()
print('DMBDT WING 2 ...  atomic crystallographics coordinates ... :')
print('  S27  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_s_27,y_s_27,z_s_27))
print('  S28  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_s_28,y_s_28,z_s_28))
print('  C29  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_29,y_c_29,z_c_29))
print('  C30  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_30,y_c_30,z_c_30))
print('  C31  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_31,y_c_31,z_c_31))
print('  C32  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_32,y_c_32,z_c_32))
print('  C33  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_33,y_c_33,z_c_33))
print('  C34  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_34,y_c_34,z_c_34))
print('  C35  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_35,y_c_35,z_c_35))
print('  C36  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_c_36,y_c_36,z_c_36))
print('  H37  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_37,y_h_37,z_h_37))
print('  H38  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_38,y_h_38,z_h_38))
print('  H39  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_39,y_h_39,z_h_39))
print('  H40  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_40,y_h_40,z_h_40))
print('  H41  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_41,y_h_41,z_h_41))
print('  H42  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_42,y_h_42,z_h_42))
print('  H43  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_43,y_h_43,z_h_43))
print('  H44  ... from DMBDT 2 : % 10.5f % 10.5f % 10.5f ' %(x_h_44,y_h_44,z_h_44))

filog.write('\n')
filog.write('[DMBDT WING 2] CRYSTALLOGRAPHIC COORDINATES OF ALL 2 ... S ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 2: S27 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_27,y_s_27,z_s_27))
filog.write('[DMBDT WING 2: S28 - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_s_28,y_s_28,z_s_28))
filog.write('\n')
filog.write('[DMBDT WING 2] CRYSTALLOGRAPHIC COORDINATES OF ALL 8 ... C ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 2: C29  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_29,y_c_29,z_c_29))
filog.write('[DMBDT WING 2: C30  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_30,y_c_30,z_c_30))
filog.write('[DMBDT WING 2: C31  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_31,y_c_31,z_c_31))
filog.write('[DMBDT WING 2: C32  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_32,y_c_32,z_c_32))
filog.write('[DMBDT WING 2: C33  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_33,y_c_33,z_c_33))
filog.write('[DMBDT WING 2: C34  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_34,y_c_34,z_c_34))
filog.write('[DMBDT WING 2: C35  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_35,y_c_35,z_c_35))
filog.write('[DMBDT WING 2: C36  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_c_36,y_c_36,z_c_36))
filog.write('\n')
filog.write('[DMBDT WING 2] CRYSTALLOGRAPHIC COORDINATES OF ALL 8 ... H ... ATOMS: ')
filog.write('\n')
filog.write('[DMBDT WING 2: H37  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_37,y_h_37,z_h_37))
filog.write('[DMBDT WING 2: H38  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_38,y_h_38,z_h_38))
filog.write('[DMBDT WING 2: H39  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_39,y_h_39,z_h_39))
filog.write('[DMBDT WING 2: H40  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_40,y_h_40,z_h_40))
filog.write('[DMBDT WING 2: H41  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_41,y_h_41,z_h_41))
filog.write('[DMBDT WING 2: H42  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_42,y_h_42,z_h_42))
filog.write('[DMBDT WING 2: H43  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_43,y_h_43,z_h_43))
filog.write('[DMBDT WING 2: H44  - x,y,z :] % 10.5f % 10.5f % 10.5f \n' %(x_h_44,y_h_44,z_h_44))

# BACK-END HEADER
print()
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *   THANK YOU ... for using our software ... FE4S4DMBDT2_MOGUL ... [v 063 - 2021-06-14]   * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *         Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]        * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *                                 Robert.Papoular@cea.fr                                  * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print()
#
filog.write(                                                                                                  '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *   THANK YOU ... for using our software ... FE4S4DMBDT2_MOGUL ... [v 063 - 2021-06-14]   * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *         Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]        * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *                                 Robert.Papoular@cea.fr                                  * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(                                                                                                  '\n')
#
#
# SET DATE AND TIME
stoptime = str(datetime.datetime.now())[:19]  # truncated after 20 characters ...
print()
print(stoptime)
print()
#
filog.write(           '\n')
filog.write(stoptime + '\n')
filog.write(           '\n')
#

# CLOSE LOGICAL FILES
filog.close()
#
filout_gsa.close()
#
filout_res.write('END \n')
filout_res.close()

            
            
