# 2021-03-07 @ BLR
# Building the ... idealized ... Tetra-Butyl-Ammonium [TBA] ... spider ... in the tetragonal unit cell - tetragonal space group : P 42/n [#56]
#

# Update LOG
#
# 2021-03-07 - Clean up time ...
# 2021-03-06 - Clean up time ...
#
# 2021-02-18 - RES [shelx] and GSA [gsas] formats are ajusted
#
# 2021-02-16 - Bug tracking in previous versions ( up to v40 )
# 2021-02-15 - Program development continues
# 2021-02-14 - Program development continues
# 2021-02-13 - Program development continues
# 2021-02-12 - Program development starts
# 


# Imported Libraries
import numpy as np
import matplotlib.pyplot as plt
import datetime

# Arrays for TBA crystallographic atomic coordinates
spider   = np.zeros((53,3))  # before pi/4 rotation
spiderr1 = np.zeros((53,3))  # after  pi/4 rotation along c-axis
spiderr2 = np.zeros((53,3))  # after  pi/2 rotation along a-axis
spideruc = np.zeros((53,3))  # after centering of the spider head

# Lists for the atomic species and names
res_species_list = []
gsa_species_list = []
gsa_names_list   = []

# Constant
space = ' '
# SHLX Constants
osf  = 1.00000
uiso = 0.00000

# SHELX / MERCURY convention : C - H - N - S - FE
type1 = 'C'
sfac1 = 1
type2 = 'H'
sfac2 = 2
type3 = 'N'
sfac3 = 3
type4 = 'S'
sfac4 = 4
type5 = 'FE'
sfac5 = 5

# theta = arccos(-1/3)
costhetao2 = np.sqrt(1.0/3.0)
sinthetao2 = np.sqrt(2.0/3.0)

# N-C , C-C bond distance : 1.51 A
# C-H       bond distance : 1.00 A
#
# p = N-C = C-C
# q = C-H
#
p  = 1.51
px = p * sinthetao2
py = p * sinthetao2
pz = p * costhetao2
#
q  = 1.00
qx = q * sinthetao2
qy = q * sinthetao2
qz = q * costhetao2
#

def xyz_rotate_45_c_axis(x,y,z):
  xr = ( x - y ) / np.sqrt(2.0)
  yr = ( x + y ) / np.sqrt(2.0)
  zr =   z
  return xr , yr , zr

def xyz_rotate_90_a_axis(x,y,z,a,b,c):
  xr =  x
  yr = -z * c / b
  zr =  y * b / c
  return xr , yr , zr

#
########### ONLY IF NECESSARY ###########
#                                       #
def dot_product(X1,Y1,Z1,X2,Y2,Z2):
  dot = X1*X2*a*a + Y1*Y2*b*b + Z1*Z2*c*c
  return dot

def cross_product(X1,Y1,Z1,X2,Y2,Z2):
  cross_x = ( Y1*Z2 - Z1*Y2 )*(b*c/a)
  cross_y = ( Z1*X2 - X1*Z2 )*(c*a/b)
  cross_z = ( X1*Y2 - Y2*X1 )*(a*b/c)
  return cross_x , cross_y , cross_z

def rotation(x1,y1,z1,x2,y2,z2,x3,y3,z3,angle):  # rotation axis : M1M2 , angle : theta [degrees ], M3 ===> M4
                                                 # M2M3 ===> M2M4
  angler = angle * np.pi / 180.0
  # AB
  u1 = x2 - x1
  v1 = y2 - y1
  w1 = z2 - z1
  # BC
  u2 = x3 - x2
  v2 = y3 - y2
  w2 = z3 - z2
  #
  ABdotBC = dot_product(u1,v1,w1,u2,v2,w2)
  #                      
  ABcrossBC_x , ABcrossBC_y , ABcrossBC_z = cross_product(u1,v1,w1,u2,v2,w2)
  #
  u1 = x2 - x1
  v1 = y2 - y1
  w1 = z2 - z1
  #
  u2 = x2 - x1
  v2 = y2 - y1
  w2 = z2 - z1
  #
  ABdotAB = dot_product(u1,v1,w1,u2,v2,w2)
  #
  x4 = x2 + (x3 - x2)   *       np.cos(angler)
  x4 = x4 + (x2 - x1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  x4 = x4 + ABcrossBC_x *       np.sin(angler)           / np.sqrt(ABdotAB)
  #
  y4 = y2 + (y3 - y2)   *       np.cos(angler)
  y4 = y4 + (y2 - y1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  y4 = y4 + ABcrossBC_y *       np.sin(angler)           / np.sqrt(ABdotAB) 
  #
  z4 = z2 + (z3 - z2)   *       np.cos(angler)
  z4 = z4 + (z2 - z1)   *(1.0 - np.cos(angler))* ABdotBC /         ABdotAB
  z4 = z4 + ABcrossBC_z *       np.sin(angler)           / np.sqrt(ABdotAB)
  return x4 , y4 , z4
#                                       #
########### ONLY IF NECESSARY ###########
#

# Open .LOG file
#
filog = open('tba1_v50.log','w')

# Open .GSA file [ my former .ATO format ]
#
filout_gsa = open('tba1.gsa','w')

# Open .RES Shelx file [ for MERCURY / CSD / CCDC visualization ]
#
filout_res = open('tba1.res','w')

# RES file header
#
res_header_1  = 'TITL P42UN_18_phase_1'
res_header_2  = 'CELL 0.45793 23.8190 23.8190 20.7652 90 90 90'
res_header_3  = 'ZERR 8 0.0005 0.0000 0.0004 0.0 0.0 0.0'
res_header_4  = 'LATT 1'
res_header_5  = 'SYMM -y,1/2+x,1/2+z'
res_header_6  = 'SYMM 1/2-x,1/2-y,z'
res_header_7  = 'SYMM 1/2+y,-x,1/2+z'
res_header_8  = 'SFAC C H N S Fe'
res_header_9  = 'UNIT 384 704 8 64 32'
res_header_10 = 'FVAR 1.00'
#
filout_res.write(res_header_1 + '\n')
filout_res.write(res_header_2 + '\n')
filout_res.write(res_header_3 + '\n')
filout_res.write(res_header_4 + '\n')
filout_res.write(res_header_5 + '\n')
filout_res.write(res_header_6 + '\n')
filout_res.write(res_header_7 + '\n')
filout_res.write(res_header_8 + '\n')

# SET DATE AND TIME
starttime = str(datetime.datetime.now())[:19]  # truncated after 20 characters ...
print()
print(starttime)
print()
#
filog.write(            '\n')
filog.write(starttime + '\n')
filog.write(            '\n')
#
#
# FRONT-END HEADER
print()
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *       WELCOME ... to our software ... TBA1_SPIDER ... [v 50 - 2021-03-07]      * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *    Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]    * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print(' *                                                                                * ')
print(' *                            Robert.Papoular@cea.fr                              * ')
print(' *                                                                                * ')
print(' ********************************************************************************** ')
print()
#
filog.write(                                                                                         '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *       WELCOME ... to our software ... TBA1_SPIDER ... [v 50 - 2021-03-07]      * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *    Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]    * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' *                            Robert.Papoular@cea.fr                              * ' + '\n')
filog.write(' *                                                                                * ' + '\n')
filog.write(' ********************************************************************************** ' + '\n')
filog.write(                                                                                         '\n')
#

#
# Lattice parameters a , b , c ... as found using a LeBail fit
# Tetragonal cell - tetragonal space group P 42/n [#56]
#
# All distances in Angstroms
#
a = 23.8190   # tetragonal unit cell
b = 23.8190   # tetragonal unit cell
c = 20.7652   # tetragonal unit cell
#
filog.write('\n')
filog.write('UNIT CELL LATTICE PARAMETERS: \n')
filog.write('\n')
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] a = % 10.5f A \n' %(a)) 
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] b = % 10.5f A \n' %(b))
filog.write('[ TETRAGONAL SPACE GROUP - P 42/n # 86 ] c = % 10.5f A \n' %(c))
filog.write('\n')
filog.write('\n')

# PART I: BUILD ... TBA1 ... SPIDER ... BEFORE THE + PI/4 ROTATION ABOUT THE C-AXIS.
#
print()
print('PART I: ... TBA1 SPIDER')
print()
#
filog.write('PART I: ... TBA1 SPIDER \n')

#
# Head of the TBA spider
# ... in crystallographic coordinates
#
x_center = 0.00   # tetragonal unit cell - tetragonal space group  
y_center = 0.25   # tetragonal unit cell - tetragonal space group
z_center = 0.25   # tetragonal unit cell - tetragonal space group
#
filog.write('\n')
filog.write('TBA1 SPIDER HEAD : N')
filog.write('\n')
filog.write('[ TBA1 spider head : N ] x_center = % 10.5f \n' %(x_center)) 
filog.write('[ TBA1 spider head : N ] y_center = % 10.5f \n' %(y_center))
filog.write('[ TBA1 spider head : N ] z_center = % 10.5f \n' %(z_center))
filog.write('\n')


#
# Head of the TBA1 spider : N
#
x_N45 = 0
y_N45 = 0
z_N45 = 0
#
spider[0][0] = x_N45
spider[0][1] = y_N45
spider[0][2] = z_N45
#
#                                     
#                                      H50   H52   H54   H57
#                                      I     I     I     I
# First leg of the TBA1 spider : N45 - C46 - C47 - C48 - C49 - H56
#                                      I     I     I     I
#                                      H51   H53   H55   H58
#
x_C46 =       px / a
y_C46 = 0.0
z_C46 =       pz / c
#
spider[1][0] = x_C46
spider[1][1] = y_C46
spider[1][2] = z_C46
#
#
x_C47 = 2.0 * px / a
y_C47 = 0.0
z_C47 = 0.0
#
spider[2][0] = x_C47
spider[2][1] = y_C47
spider[2][2] = z_C47
#
#
x_C48 = 3.0 * px / a
y_C48 = 0.0
z_C48 =       pz / c
#
spider[3][0] = x_C48
spider[3][1] = y_C48
spider[3][2] = z_C48
#
#
x_C49 = 4.0 * px / a
y_C49 = 0.0
z_C49 = 0.0
#
spider[4][0] = x_C49
spider[4][1] = y_C49
spider[4][2] = z_C49
#
#
x_H50 = x_C46
y_H50 = y_C46 + qy / b
z_H50 = z_C46 + qz / c
#
spider[5][0] = x_H50
spider[5][1] = y_H50
spider[5][2] = z_H50
#
#
x_H51 = x_C46
y_H51 = y_C46 - qy / b
z_H51 = z_C46 + qz / c
#
spider[6][0] = x_H51
spider[6][1] = y_H51
spider[6][2] = z_H51
#
#
x_H52 = x_C47
y_H52 = y_C47 + qy / b
z_H52 = z_C47 - qz / c
#
spider[7][0] = x_H52
spider[7][1] = y_H52
spider[7][2] = z_H52
#
#
x_H53 = x_C47
y_H53 = y_C47 - qy / b
z_H53 = z_C47 - qz / c
#
spider[8][0] = x_H53
spider[8][1] = y_H53
spider[8][2] = z_H53
#
#
x_H54 = x_C48
y_H54 = y_C48 + qy / b
z_H54 = z_C48 + qz / c
#
spider[9][0] = x_H54
spider[9][1] = y_H54
spider[9][2] = z_H54
#
#
x_H55 = x_C48
y_H55 = y_C48 - qy / b
z_H55 = z_C48 + qz / c
#
spider[10][0] = x_H55
spider[10][1] = y_H55
spider[10][2] = z_H55
#
#
x_H56 = x_C49 + qx / b
y_H56 = y_C49
z_H56 = z_C49 + qz / c
#
spider[11][0] = x_H56
spider[11][1] = y_H56
spider[11][2] = z_H56
#
#
x_H57 = x_C49
y_H57 = y_C49 + qy / b
z_H57 = z_C49 - qz / c
#
spider[12][0] = x_H57
spider[12][1] = y_H57
spider[12][2] = z_H57
#
x_H58 = x_C49
y_H58 = y_C49 - qy / b
z_H58 = z_C49 - qz / c
#
spider[13][0] = x_H58
spider[13][1] = y_H58
spider[13][2] = z_H58
#
#

#
#                                     
#                                       H63   H65   H67   H70
#                                       I     I     I     I
# Second leg of the TBA1 spider : N45 - C59 - C60 - C61 - C62 - H69
#                                       I     I     I     I
#                                       H64   H66   H68   H71
#
x_C59 =      - px / a
y_C59 =  0.0
z_C59 =        pz / c
#
spider[14][0] = x_C59
spider[14][1] = y_C59
spider[14][2] = z_C59
#
#
x_C60 = -2.0 * px / a
y_C60 =  0.0
z_C60 =  0.0
#
spider[15][0] = x_C60
spider[15][1] = y_C60
spider[15][2] = z_C60
#
#
x_C61 = -3.0 * px / a
y_C61 =  0.0
z_C61 =        pz / c
#
spider[16][0] = x_C61
spider[16][1] = y_C61
spider[16][2] = z_C61
#
#
x_C62 = -4.0 * px / a
y_C62 =  0.0
z_C62 =  0.0
#
spider[17][0] = x_C62
spider[17][1] = y_C62
spider[17][2] = z_C62
#
#
x_H63 = x_C59
y_H63 = y_C59 + qy / b
z_H63 = z_C59 + qz / c
#
spider[18][0] = x_H63
spider[18][1] = y_H63
spider[18][2] = z_H63
#
#
x_H64 = x_C59
y_H64 = y_C59 - qy / b
z_H64 = z_C59 + qz / c
#
spider[19][0] = x_H64
spider[19][1] = y_H64
spider[19][2] = z_H64
#
#
x_H65 = x_C60
y_H65 = y_C60 + qy / b
z_H65 = z_C60 - qz / c
#
spider[20][0] = x_H65
spider[20][1] = y_H65
spider[20][2] = z_H65
#
#
x_H66 = x_C60
y_H66 = y_C60 - qy / b
z_H66 = z_C60 - qz / c
#
spider[21][0] = x_H66
spider[21][1] = y_H66
spider[21][2] = z_H66
#
#
x_H67 = x_C61
y_H67 = y_C61 + qy / b
z_H67 = z_C61 + qz / c
#
spider[22][0] = x_H67
spider[22][1] = y_H67
spider[22][2] = z_H67
#
#
x_H68 = x_C61
y_H68 = y_C61 - qy / b
z_H68 = z_C61 + qz / c
#
spider[23][0] = x_H68
spider[23][1] = y_H68
spider[23][2] = z_H68
#
#
x_H69 = x_C62 - qx / a
y_H69 = y_C62
z_H69 = z_C62 + qz / c
#
spider[24][0] = x_H69
spider[24][1] = y_H69
spider[24][2] = z_H69
#
#
x_H70 = x_C62
y_H70 = y_C62 + qy / b
z_H70 = z_C62 - qz / c
#
spider[25][0] = x_H70
spider[25][1] = y_H70
spider[25][2] = z_H70
#
#
x_H71 = x_C62
y_H71 = y_C62 - qy / b
z_H71 = z_C62 - qz / c
#
spider[26][0] = x_H71
spider[26][1] = y_H71
spider[26][2] = z_H71
#
#

#                                     
#                                      H76   H78   H80   H83
#                                      I     I     I     I
# Third leg of the TBA1 spider : N45 - C72 - C73 - C74 - C75 - H82
#                                      I     I     I     I
#                                      H77   H79   H81   H84
#
x_C72 = 0.0
y_C72 =       py / b
z_C72 =     - pz / c
#
spider[27][0] = x_C72
spider[27][1] = y_C72
spider[27][2] = z_C72
#
#
x_C73 = 0.0
y_C73 = 2.0 * py / b
z_C73 = 0.0
#
spider[28][0] = x_C73
spider[28][1] = y_C73
spider[28][2] = z_C73
#
#
x_C74 = 0.0
y_C74 = 3.0 * py / b
z_C74 =     - pz / c
#
spider[29][0] = x_C74
spider[29][1] = y_C74
spider[29][2] = z_C74
#
#
x_C75 = 0.0
y_C75 = 4.0 * py / b
z_C75 = 0.0
#
spider[30][0] = x_C75
spider[30][1] = y_C75
spider[30][2] = z_C75
#
#
x_H76 = x_C72 + qx / a
y_H76 = y_C72
z_H76 = z_C72 - qz / c
#
spider[31][0] = x_H76
spider[31][1] = y_H76
spider[31][2] = z_H76
#
#
x_H77 = x_C72 - qx / a
y_H77 = y_C72
z_H77 = z_C72 - qz / c
#
spider[32][0] = x_H77
spider[32][1] = y_H77
spider[32][2] = z_H77
#
#
x_H78 = x_C73 + qx / a
y_H78 = y_C73
z_H78 = z_C73 + qz / c
#
spider[33][0] = x_H78
spider[33][1] = y_H78
spider[33][2] = z_H78
#
#
x_H79 = x_C73 - qx / a
y_H79 = y_C73
z_H79 = z_C73 + qz / c
#
spider[34][0] = x_H79
spider[34][1] = y_H79
spider[34][2] = z_H79
#
#
x_H80 = x_C74 + qx / a
y_H80 = y_C74
z_H80 = z_C74 - qz / c
#
spider[35][0] = x_H80
spider[35][1] = y_H80
spider[35][2] = z_H80
#
#
x_H81 = x_C74 - qx / a
y_H81 = y_C74
z_H81 = z_C74 - qz / c
#
spider[36][0] = x_H81
spider[36][1] = y_H81
spider[36][2] = z_H81
#
#
x_H82 = x_C75
y_H82 = y_C75 + qy / b
z_H82 = z_C75 - qz / c
#
spider[37][0] = x_H82
spider[37][1] = y_H82
spider[37][2] = z_H82
#
#
x_H83 = x_C75 + qx / a
y_H83 = y_C75
z_H83 = z_C75 + qz / c
#
spider[38][0] = x_H83
spider[38][1] = y_H83
spider[38][2] = z_H83
#
x_H84 = x_C75 - qx / a
y_H84 = y_C75
z_H84 = z_C75 + qz / c
#
spider[39][0] = x_H84
spider[39][1] = y_H84
spider[39][2] = z_H84
#
#

#                                     
#                                       H89   H91   H93   H96
#                                       I     I     I     I
# Fourth leg of the TBA1 spider : N45 - C85 - C86 - C87 - C88 - H95
#                                       I     I     I     I
#                                       H90   H92   H94   H97
#
x_C85 =   0.0
y_C85 =       - py / b
z_C85 =       - pz / c
#
spider[40][0] = x_C85
spider[40][1] = y_C85
spider[40][2] = z_C85
#
#
x_C86 =   0.0
y_C86 = - 2.0 * py / a
z_C86 =   0.0
#
spider[41][0] = x_C86
spider[41][1] = y_C86
spider[41][2] = z_C86
#
#
x_C87 =   0.0
y_C87 = - 3.0 * py / b
z_C87 =       - pz / c
#
spider[42][0] = x_C87
spider[42][1] = y_C87
spider[42][2] = z_C87
#
#
x_C88 =   0.0
y_C88 = - 4.0 * py / b
z_C88 =   0.0
#
spider[43][0] = x_C88
spider[43][1] = y_C88
spider[43][2] = z_C88
#
#
x_H89 = x_C85 + qx / a
y_H89 = y_C85
z_H89 = z_C85 - qz / c
#
spider[44][0] = x_H89
spider[44][1] = y_H89
spider[44][2] = z_H89
#
#
x_H90 = x_C85 - qx / a
y_H90 = y_C85
z_H90 = z_C85 - qz / c
#
spider[45][0] = x_H90
spider[45][1] = y_H90
spider[45][2] = z_H90
#
#
x_H91 = x_C86 + qx / a
y_H91 = y_C86
z_H91 = z_C86 + qz / c
#
spider[46][0] = x_H91
spider[46][1] = y_H91
spider[46][2] = z_H91
#
#
x_H92 = x_C86 - qx / a
y_H92 = y_C86
z_H92 = z_C86 + qz / c
#
spider[47][0] = x_H92
spider[47][1] = y_H92
spider[47][2] = z_H92
#
#
x_H93 = x_C87 + qx / a
y_H93 = y_C87
z_H93 = z_C87 - qz / c
#
spider[48][0] = x_H93
spider[48][1] = y_H93
spider[48][2] = z_H93
#
#
x_H94 = x_C87 - qx / a
y_H94 = y_C87
z_H94 = z_C87 - qz / c
#
spider[49][0] = x_H94
spider[49][1] = y_H94
spider[49][2] = z_H94
#
#
x_H95 = x_C88
y_H95 = y_C88 - qy / b
z_H95 = z_C88 - qz / c
#
spider[50][0] = x_H95
spider[50][1] = y_H95
spider[50][2] = z_H95
#
#
x_H96 = x_C88 + qx / a
y_H96 = y_C88
z_H96 = z_C88 + qz / c
#
spider[51][0] = x_H96
spider[51][1] = y_H96
spider[51][2] = z_H96
#
x_H97 = x_C88 - qx / a
y_H97 = y_C88
z_H97 = z_C88 + qz / c
#
spider[52][0] = x_H97
spider[52][1] = y_H97
spider[52][2] = z_H97
#
#
print('PART I: ... done')
print()
#
filog.write('PART I: ... done \n')
filog.write('                 \n')

#
# PART II: ROTATE TBA1 SPIDER BY 45 AND TRANSLATE IT
#
print()
print('PART II: ... ROTATE AND TRANSLATE ... TBA1 SPIDER')
print()
#
filog.write('PART II: ... ROTATE AND TRANSLATE ... TBA1 SPIDER \n')
filog.write('                                                 \n')

count = 0
while(count < 53):
  count = count + 1  
  print('Atom % 2d ... has just been rotated ...' %(count))
  #
  # Chemical species
  #
  if count in range(2):
    print(      'This is the ... N ... atom.')
    filog.write('This is the ... N ... atom. \n')
    #
    res_species_list.append(sfac3)
    sfac = sfac3
    gsa_species_list.append(type3)
    atom_type = type3.ljust(2)
    atom_name = type3 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(2,6 ):
    print(      'This is a ... C ... atom from leg 1.'   )
    filog.write('This is a ... C ... atom from leg 1. \n')
    #
    res_species_list.append(sfac1)
    sfac = sfac1
    gsa_species_list.append(type1)
    atom_type = type1.ljust(2)
    atom_name = type1 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(6,15):
    print(      'This is a ... H ... atom from leg 1.'   )
    filog.write('This is a ... H ... atom from leg 1. \n')
    #
    res_species_list.append(sfac2)
    sfac = sfac2
    gsa_species_list.append(type2)
    atom_type = type2.ljust(2)
    atom_name = type2 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(15,19):
    print(      'This is a ... C ... atom from leg 2.'   )
    filog.write('This is a ... C ... atom from leg 2. \n')
    #
    res_species_list.append(sfac1)
    sfac = sfac1
    gsa_species_list.append(type1)
    atom_type = type1.ljust(2)
    atom_name = type1 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(19,28):
    print(      'This is a ... H ... atom from leg 2.'   )
    filog.write('This is a ... H ... atom from leg 2. \n')
    #
    res_species_list.append(sfac2)
    sfac = sfac2
    gsa_species_list.append(type2)
    atom_type = type2.ljust(2)
    atom_name = type2 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(28,32):
    print(      'This is a ... C ... atom from leg 3.'   )
    filog.write('This is a ... C ... atom from leg 3. \n')
    #
    res_species_list.append(sfac1)
    sfac = sfac1
    gsa_species_list.append(type1)
    atom_type = type1.ljust(2)
    atom_name = type1 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(32,41):
    print(      'This is a ... H ... atom from leg 3.'   )
    filog.write('This is a ... H ... atom from leg 3. \n')
    #
    res_species_list.append(sfac2)
    sfac = sfac2
    gsa_species_list.append(type2)
    atom_type = type2.ljust(2)
    atom_name = type2 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(41,45):
    print(      'This is a ... C ... atom from leg 4.'   )
    filog.write('This is a ... C ... atom from leg 4. \n')
    #
    res_species_list.append(sfac1)
    sfac = sfac1
    gsa_species_list.append(type1)
    atom_type = type1.ljust(2)
    atom_name = type1 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  elif count in range(45,54):
    print(      'This is a ... H ... atom from leg 4.'   )
    filog.write('This is a ... H ... atom from leg 4. \n')
    #
    res_species_list.append(sfac2)
    sfac = sfac2
    gsa_species_list.append(type2)
    atom_type = type2.ljust(2)
    atom_name = type2 + str(count + 44)
    atom_name = atom_name.ljust(6)
    gsa_names_list.append(atom_name)
    #
    print(      'Its full ... GSAS ... name ... is : ... % 4s   ' %(atom_name))
    filog.write('Its full ... GSAS ... name ... is : ... % 4s \n' %(atom_name))
  #
  x = spider[count-1][0]
  y = spider[count-1][1]
  z = spider[count-1][2]
  #
  xr1,yr1,zr1 = xyz_rotate_45_c_axis(x,y,z)
  xr1f = float(xr1)
  yr1f = float(yr1)
  zr1f = float(zr1)
  #
  spiderr1[count-1][0] = xr1f
  spiderr1[count-1][1] = yr1f
  spiderr1[count-1][2] = zr1f
  #
  xr2,yr2,zr2 = xyz_rotate_90_a_axis(xr1f,yr1f,zr1f,a,b,c)
  xr2f = float(xr2)
  yr2f = float(yr2)
  zr2f = float(zr2)
  #
  spiderr2[count-1][0] = xr2f
  spiderr2[count-1][1] = yr2f
  spiderr2[count-1][2] = zr2f
  #
  xuc = xr2f + x_center
  yuc = yr2f + y_center
  zuc = zr2f + z_center
  #
  spideruc[count-1][0] = xuc
  spideruc[count-1][1] = yuc
  spideruc[count-1][2] = zuc
  #
  print('Atom % 2d ... has just been rotated ...       \n' %(count           ))
  print('  Before rotation    ... % 10.5f % 10.5f % 10.5f' %(x    ,y    ,z   ))
  print('  After  rotation    ... % 10.5f % 10.5f % 10.5f' %(xr2f ,yr2f ,zr2f))
  print('  After  translation ... % 10.5f % 10.5f % 10.5f' %(xuc,yuc,zuc     )) 
  print()
  # 
  filog.write('Atom % 2d ... has just been rotated ...          \n' %(count         )) 
  filog.write('  Before rotation    ... % 10.5f % 10.5f % 10.5f \n' %(x   ,y   ,z   ))
  filog.write('  After  rotation    ... % 10.5f % 10.5f % 10.5f \n' %(xr2f,yr2f,zr2f))
  filog.write('  After  translation ... % 10.5f % 10.5f % 10.5f \n' %(xuc,yuc,zuc   ))
  filog.write('                                                 \n'                  )
  #
  filout_res.write('%6s%3s%10.6f%10.6f%10.6f%10.6f%9.6f\n' %(atom_name,sfac,xuc,yuc,zuc,osf,uiso))
  filout_gsa.write('%2s%10.6f%10.6f%10.6f%10.6f%1s%6s%9.6f\n' %(atom_type,xuc,yuc,zuc,osf,space,atom_name,uiso))  
print()
print('PART II: ... done')
print()
#
filog.write('PART II: ... done \n')
filog.write('                  \n')

print()
print(res_species_list)
print()
print(gsa_species_list)
print()
print(gsa_names_list)
print()
  
# BACK-END HEADER
print()
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *       THANK YOU ... for using our software ... TBA1_SPIDER ... [v 50 - 2021-03-07]      * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *         Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]        * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print(' *                                                                                         * ')
print(' *                                 Robert.Papoular@cea.fr                                  * ')
print(' *                                                                                         * ')
print(' ******************************************************************************************* ')
print()
#
filog.write(                                                                                                  '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *       THANK YOU ... for using our software ... TBA1_SPIDER ... [v 50 - 2021-03-07]      * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *         Written by ... Robert J PAPOULAR ... [IRAMIS, DRF , CEA-Saclay , France]        * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' *                                 Robert.Papoular@cea.fr                                  * ' + '\n')
filog.write(' *                                                                                         * ' + '\n')
filog.write(' ******************************************************************************************* ' + '\n')
filog.write(                                                                                                  '\n')
#
#
# SET DATE AND TIME
stoptime = str(datetime.datetime.now())[:19]  # truncated after 20 characters ...
print()
print(stoptime)
print()
#
filog.write(           '\n')
filog.write(stoptime + '\n')
filog.write(           '\n')
#

# CLOSE LOGICAL FILES
filog.close()
#
filout_gsa.close()
#
filout_res.write('END \n')
filout_res.close()

            
            
